import { executeComposer } from "./execute.js";
import * as remoteControl from "./remoteControl.js";

/**
 * @typedef {Record<string, import("./options.js").Question>} ArgType
 */

/**
 * @template {ArgType} Args
 * @typedef {import("../utils/workspace").Workspace<Args>} Workspace
 */

/**
 * @template {ArgType} Args
 * @typedef  {{
 *   [K in keyof Args]: Args[K]["type"] extends "string" ? string: Args[K]["type"] extends "boolean"? boolean: Args[K]["type"] extends "number"? number: never
 * }} ArgValues
 */

/**
 * @template {ArgType} Args
 * @typedef PackageDefinition
 * @property {string} name
 * @property {string} version
 * @property {boolean} dev
 * @property {ConditionDefinition<Args>} [condition]
 */

/**
 * @template {ArgType} Args
 * @typedef {(workspace: Workspace<Args>) => boolean} ConditionDefinition
 */

/**
 * @template {ArgType} Args
 * @typedef {import("../files/processors").ScriptFile<Args>
 *  | import("../files/processors").TextFile<Args>
 *  | import("../files/processors").SvelteFile<Args>
 *  | import("../files/processors").JsonFile<Args>
 *  | import("../files/processors").HtmlFile<Args>
 *  | import("../files/processors").CssFile<Args>
 * } FileTypes
 */

/**
 * @typedef WebsiteMetadata
 * @property {string} website.logo
 * @property {string[]} website.keywords
 * @property {string} documentation
 */

/**
 * @typedef ComposerConfigEnvironments
 * @property {boolean} svelte
 * @property {boolean} kit
 */

/**
 * @typedef ComposerConfigMetadata
 * @property {string} id
 * @property {string} package
 * @property {string} version
 * @property {string} name
 * @property {string} description
 * @property {import("./categories.js").CategoryInfo} category
 * @property {ComposerConfigEnvironments} environments
 * @property {WebsiteMetadata} [website]
 */

/**
 * @template {ArgType} Args
 * @typedef ComposerConfig
 * @property {ComposerConfigMetadata} metadata
 * @property {Args?} options
 * @property {PackageDefinition<Args>[]} packages
 * @property {FileTypes<Args>[]} files
 * @property {(options: Workspace<Args>) => Promise<void>} [installHook]
 * @property {(options: Workspace<Args>) => Promise<void>} [uninstallHook]
 */

/**
 * @template {ArgType} Args
 * @param {ComposerConfig<Args>} config
 * @returns {ComposerConfig<Args>}
 */
export function defineComposerConfig(config) {
    return config;
}

/**
 * @template {ArgType} Args
 * @typedef Composer
 * @property {ComposerConfig<Args>} config
 * @property {ComposerCheckConfig<Args>} checks
 * @property {ComposerTestConfig<Args>} [tests]
 */

/**
 * @typedef {Composer<Record<string, import("./options.js").Question>>} ComposerWithoutExplicitArgs
 */

/**
 * @template {ArgType} Args
 * @param {ComposerConfig<Args>} config
 * @param {ComposerCheckConfig<Args>} checks
 * @param {ComposerTestConfig<Args>} [tests]
 * @returns {Composer<Args>}
 */
export function defineComposer(config, checks, tests) {
    const remoteControlled = remoteControl.isRemoteControlled();
    if (!remoteControlled) {
        executeComposer(config, checks);
    }

    return { config, checks, tests };
}

/**
 * @typedef Tests
 * @property {(selector: string, property: string, expectedValue: string) => Promise<void>} expectProperty
 * @property {(selector: string) => Promise<void>} elementExists
 * @property {(selector: string, waitForNavigation: boolean) => Promise<void>} click
 * @property {(path: string) => Promise<void>} expectUrlPath
 */

/**
 * @template {ArgType} Args
 * @typedef TestDefinition
 * @property {string} name
 * @property {(tests: Tests) => Promise<void>} run
 * @property {(options: ArgValues<Args>) => boolean} [condition]
 */

/**
 * @template {ArgType} Args
 * @typedef {{
 *   [K in keyof Args]: boolean | number | string
 * }} OptionValues
 */

/**
 * @template {ArgType} Args
 * @typedef ComposerTestConfig
 * @property {FileTypes<Args>[]} files
 * @property {TestDefinition<Args>[]} tests
 * @property {Args} options
 * @property {ArgValues<Args>[]} optionValues
 */

/**
 * @template {ArgType} Args
 * @param {ComposerTestConfig<Args>} tests
 * @returns {ComposerTestConfig<Args>}
 */
export function defineComposerTests(tests) {
    return tests;
}

/**
 * @template {ArgType} Args
 * @param {Args} options
 * @returns {Args}
 */
export function defineComposerOptions(options) {
    return options;
}

/**
 * @typedef PreInstallationCheck
 * @property {string} name
 * @property {(workingDirectory: string) => boolean} run
 */

/**
 * @template {ArgType} Args
 * @typedef PostInstallationCheck
 * @property {string} name
 * @property {(workspace: Workspace<Args>) => boolean} run
 */

/**
 * @template {ArgType} Args
 * @typedef ComposerCheckConfig
 * @property {PreInstallationCheck[]} [preInstallation]
 * @property {PostInstallationCheck<Args>[]} postInstallation
 * @property {Args} options
 */

/**
 * @template {ArgType} Args
 * @param {ComposerCheckConfig<Args>} checks
 * @returns {ComposerCheckConfig<Args>}
 */
export function defineComposerChecks(checks) {
    return checks;
}
