import { commonFilePaths, readFile } from "../files/utils.js";

/**
 * @typedef Package
 * @property {string} name
 * @property {string} version
 * @property {Record<string, string>} dependencies
 * @property {Record<string, string>} devDependencies
 */

/**
 *
 * @template {import("../composer/config").ArgType} Args
 * @param {import("../composer/config").Workspace<Args>} workspace
 * @returns {Promise<Package>}
 */
export async function getPackageJson(workspace) {
    const packageText = await readFile(workspace, commonFilePaths.packageJsonFilePath);
    if (!packageText) {
        /** @type {Package} */
        return {
            dependencies: {},
            devDependencies: {},
            name: "",
            version: "",
        };
    }

    /** @type {Package} */
    const packageJson = JSON.parse(packageText);
    return packageJson;
}
