import { cancel, intro, isCancel, outro, select, text, multiselect } from "@clack/prompts";

/**
 * @typedef PromptOption
 * @property {string} label
 * @property {boolean | string} value
 * @property {string} [hint]
 */

/**
 *
 * @param {string} message
 */
export function startPrompts(message) {
    intro(message);
}

/**
 *
 * @param {string} message
 */
export function endPrompts(message) {
    outro(message);
}

/**
 *
 * @param {string} question
 * @param {boolean} initialValue
 * @returns
 */
export async function booleanPrompt(question, initialValue) {
    return selectPrompt(question, initialValue, [
        { label: "Yes", value: true },
        { label: "No", value: false },
    ]);
}

/**
 *
 * @param {string} question
 * @param {boolean | string} initialValue
 * @param {PromptOption[]} options
 * @returns {Promise<string | boolean>}
 */
export async function selectPrompt(question, initialValue, options) {
    const value = await select({
        message: question,
        options,
        initialValue: initialValue,
    });

    return /** @type {string | boolean} */ (cancelIfRequired(value));
}

/**
 *
 * @param {string} question
 * @param {string} [placeholder=""]
 * @param {string} [initialValue=""]
 * @returns
 */
export async function textPrompt(question, placeholder = "", initialValue = "") {
    const value = await text({
        message: question,
        placeholder,
        initialValue,
    });

    /** @type {string} */
    const result = /** @type {any} */ (cancelIfRequired(value));
    return result;
}

/**
 *
 * @param {string} question
 * @param {PromptOption[]} options
 * @returns {Promise<string[]>}
 */
export async function multiSelectPrompt(question, options) {
    const value = await multiselect({
        message: question,
        options,
        required: false,
    });

    return /** @type {string[]} */ (cancelIfRequired(value));
}

/**
 *
 * @param {string | string[] | boolean | symbol} value
 * @returns {string | string[] | boolean}
 */
function cancelIfRequired(value) {
    if (typeof value == "symbol" || isCancel(value)) {
        cancel("Operation cancelled.");
        process.exit(0);
    }

    return value;
}
